#include <iostream>
#include <algorithm>
#include <string>
#include <sstream>

using namespace std;

class Money {
public:
    Money(long dollars, int cents);
    Money(long dollars);
    Money();

    double get_value() const;

    friend Money operator+(const Money &amount1, const Money &amount2);
    friend Money operator-(const Money &amount1, const Money &amount2);
    friend Money operator-(const Money &amount);
    friend bool operator==(const Money &amount1, const Money &amount2);
    Money &operator+=(const Money &rhs);
    Money &operator-=(const Money &rhs);
    friend istream &operator>>(istream &ins, Money &amount);
    friend ostream &operator<<(ostream &outs, const Money &amount);

private:
    long all_cents;
};

class Check {
private:
    int number;
    Money amount;
    bool cashed;
public:
    explicit Check(int m1 = 0, Money m2 = Money(), bool m3 = false);

    int get_number() const;
    Money get_amount() const;
    bool get_cashed() const;

    void set_number(int val);
    void set_amount(Money new_money);
    void set_cashed(bool is_cashed);

    Check operator+(Check &other);
    friend istream &operator>>(istream &ins, Check &my_check);
    friend ostream &operator<<(ostream &outs, Check &my_check);
    bool operator<(const Check &other) const;
};

void service_check(Money &sum_cashed_checks, Money &sum_uncashed_checks);

void service_deposit(Money &sum_deposits);

void service_balance(Money &sum_cashed_checks, Money &sum_deposits);

int main() {
    cout << "Welcome to our banking system!\n";
    Money sum_cashed_checks, sum_uncashed_checks;
    Money sum_deposits;

    service_check(sum_cashed_checks, sum_uncashed_checks);
    service_deposit(sum_deposits);
    service_balance(sum_cashed_checks, sum_deposits);
    return 0;
}


int digit_to_int(char c) {
    return static_cast<int>(c) - static_cast<int>('0');
}

Money::Money(long dollars, int cents) {
    if (dollars * cents < 0) {
        cout << "Illegal values for dollars and cents.\n";
        exit(1);
    }

    all_cents = dollars * 100 + cents;
}

Money::Money(long dollars) : all_cents(dollars * 100) {}

Money::Money() : all_cents(0) {}

double Money::get_value() const {
    return all_cents * 0.01;
}

Money operator+(const Money &amount1, const Money &amount2) {
    Money temp;
    temp.all_cents = amount1.all_cents + amount2.all_cents;

    return temp;
}

Money operator-(const Money &amount1, const Money &amount2) {
    Money temp;
    temp.all_cents = amount1.all_cents - amount2.all_cents;

    return temp;
}

Money operator-(const Money &amount) {
    Money temp;
    temp.all_cents = -amount.all_cents;

    return temp;
}

bool operator==(const Money &amount1, const Money &amount2) {
    return amount1.all_cents == amount2.all_cents;
}

Money &Money::operator+=(const Money &rhs) {
    this->all_cents += rhs.all_cents;
    return *this;
}

Money &Money::operator-=(const Money &rhs) {
    this->all_cents -= rhs.all_cents;
    return *this;
}

istream &operator>>(istream &ins, Money &amount) {
    char one_char, decimal_point, digit1, digit2;
    long dollars;
    int cents;
    bool negative;

    ins >> one_char;

    if (one_char == '-') {
        negative = true;
        ins >> one_char; // read '$'
    } else {
        negative = false;
    }

    ins >> dollars >> decimal_point >> digit1 >> digit2;

    if (one_char != '$' || decimal_point != '.' || !isdigit(digit1) || !isdigit(digit2)) {
        cout << "Error illegal form for money input\n";
        exit(1);
    }

    cents = digit_to_int(digit1) * 10 + digit_to_int(digit2);
    amount.all_cents = dollars * 100 + cents;

    if (negative) {
        amount.all_cents = -amount.all_cents;
    }

    return ins;
}

ostream &operator<<(ostream &outs, const Money &amount) {
    long positive_cents, dollars, cents;
    positive_cents = labs(amount.all_cents);
    dollars = positive_cents / 100;
    cents = positive_cents % 100;

    if (amount.all_cents < 0) {
        outs << "-$" << dollars << '.';
    } else {
        outs << '$' << dollars << '.';
    }

    if (cents < 10) { outs << '0'; }
    outs << cents;
    return outs;
}

Check::Check(int m1, Money m2, bool m3) {
    number = m1;
    amount = m2;
    cashed = m3;
}

int Check::get_number() const { return number; }

Money Check::get_amount() const { return amount; }

bool Check::get_cashed() const { return cashed; }

void Check::set_number(int val) { number = val; }

void Check::set_amount(Money new_money) { amount = new_money; }

void Check::set_cashed(bool is_cashed) { cashed = is_cashed; }

istream &operator>>(istream &ins, Check &my_check) {
    string temp_cashed;
    ins >> my_check.number >> my_check.amount >> temp_cashed;

    if (temp_cashed == "1") {
        my_check.cashed = true;
    } else if (temp_cashed == "0") {
        my_check.cashed = false;
    } else {
        cout << "Invalid input.\n";
        exit(1);
    }

    return ins;
}

ostream &operator<<(ostream &outs, Check &my_check) {
    outs << "Check #" << my_check.number;
    outs << (my_check.cashed ? " is cashed " : " hasn't been cashed ");
    outs << "with an amount of $" << my_check.amount << endl;
    return outs;
}

Check Check::operator+(Check &other) {
    Check temp(*this);
    temp.amount += other.amount;
    return temp;
}

bool Check::operator<(const Check &other) const {
    return (this->number < other.number);
}

void
service_check(Money &sum_cashed_checks, Money &sum_uncashed_checks) {
    cout << "Enter the number of checks, eg. 2 for 2 checks:\n";
    int number_of_checks;
    cin >> number_of_checks;
    Check *list_of_checks = new Check[number_of_checks];

    cout    << "Please enter each check in one line with the information about number, amount(in dollars) and whether it's been cashed or not.\n"
            << "Separated by a space,\n"
               "number is only valid with inputting digits,\n"
               "amount should be in the format of $##.## (which means max is 2 decimal points), eg. $12.34\n"
               "1 for cashed, 0 for un-cashed\n"
            << "Example input: 1 $23.45 0\n"
               "1 for number, $23.45 for amount and 0 for being un-cashed\n";

    for (int i = 0; i < number_of_checks; ++i) {
        cin >> list_of_checks[i];
        if (list_of_checks[i].get_cashed()) {
            sum_cashed_checks += list_of_checks[i].get_amount();

        } else {
            sum_uncashed_checks += list_of_checks[i].get_amount();
        }
    }
    sort(list_of_checks, list_of_checks + number_of_checks);

    cout << "\n\n";
    cout << "The sum of the cashed checks is " << sum_cashed_checks << endl;
    cout << "The sum of the uncashed checks is " << sum_uncashed_checks << endl;


    cout << "\n\n";
    if (sum_cashed_checks.get_value() > 0) {
        cout << "The cashed checks are:\n";
        for (int i = 0; i < number_of_checks; ++i) {
            Check temp = list_of_checks[i];
            if (temp.get_cashed())
                cout << list_of_checks[i];
        }
    } else cout << "There is no cashed check.\n";

    cout << "\n\n";
    if (sum_uncashed_checks.get_value() > 0) {
        cout << "The uncashed checks are:\n";
        for (int i = 0; i < number_of_checks; ++i) {
            Check temp = list_of_checks[i];
            if (!temp.get_cashed())
                cout << list_of_checks[i];
        }
    } else cout << "There is no un-cashed check.\n";

    delete[] list_of_checks;
    list_of_checks = nullptr;
}


void service_deposit(Money &sum_deposits) {
    cout << "\n\nPlease enter the number of deposits, eg. 2 for 2 deposits:\n";
    int num_of_deposits;
    cin >> num_of_deposits;
    cout << "Please enter the exact amount of each deposit(in dollars), one deposit in one line with format $##.##(which means max is 2 decimal points:\n";
    for (int i = 0; i < num_of_deposits; ++i) {
        Money temp;
        cin >> temp;
        sum_deposits += temp;
    }
    cout << "\n\nThe sum of the deposits is " << sum_deposits << endl;
}

void service_balance(Money &sum_cashed_checks, Money &sum_deposits) {
    cout << "\n\nPlease enter the prior balance amount(format $##.##):\n";
    Money prior_balance;
    cin >> prior_balance;
    cout << "\nPlease enter the balance according to account holder($##.##):\n";
    Money holder_balance;
    cin >>  holder_balance;
    Money new_balance;
    cout << "\n\n";

    new_balance = prior_balance + sum_deposits - sum_cashed_checks;

    cout << "The new balance now is " << new_balance << endl;
    cout << "The difference between the account holder's balance of " << holder_balance
         << " and the new bank balance of " << new_balance
         << " is: " << new_balance - holder_balance << endl;
}

